#ifndef _ALLARCH_H
#define _ALLARCH_H

/*
	Constants shared by all architectures
*/

/*
 * Naming convention _P at the end of constant name means,
 * that this constant is in "pointers". 
 * When used, should be multiplied by POINTER_SIZE of concrete architecture
 */


#define GB_1 1099511627776LLU	// 1 GB
#define MB_4 4194304		// 4 MB
#define MB_2 2097152		// 2 MB
#define MB_1 1048576		// 1 MB
#define KB_64 65536		// 64 KB
#define KB_4 4096		// 4 KB
#define KB_1 1024		// 1 KB

//definitions of 
#define OS_UNKNOWN	0
#define OS_LINUX	1
#define OS_WINDOWS	2
#define OS_CIRC_HL	10	// any OS "bellow" 10, important is list_head type
#define OS_BSD		11


// MAGIC section...
// Linux specials

#define LINUX_MAGIC_PROCESS "kthread\0" 
// process, that should be found in all 2.6 running linuxes.
// Can be changed by parameter --magic-process

#define LINUX_MAGIC_USER_PROCESS "init\0"
// user process, that should be found in all 2.6 running linuxes.
// Can be changed by parameter --magic-user-process
// We need this, when looking for task_struct that have valid userspace PGD

#define LINUX_MAGIC_MODULE "yenta_socket"
// module, that should be found in all 2.6 running linuxes.
//(if modules are enabled) I have not found module present on ALL systems, 
// probably you will need to specify this by parameter
// can be changed by parameter --magic-module

#define WIN_MAGIC_PROCESS "svchost.exe" 
// process, that should be found in all running windows systems.
// Can be changed by parameter --magic-process (same as LINUX_MAGIC_PROCES)
// The logic behind this is: 
// if not specified, dump is considered to be linux, 
// if looking for linux process fail, dump is considered to be windows image 
// and search starts again


// ----- task_struct -----
#define TS_MAGIC_HEADLIST_NUMBER 3 // 9 XXX
//10//distance in list_head occurrences between "comm" and "tasks" in task_struct
// TODO add specification by parameter ? 
 
#define TS_MAGIC_HEADLIST_FRONT_NUMBER 5 //3 // Because backward search does not work good enough
#define TS_MAGIC_HEADLIST_SPREAD 16 // 6 XXX
// How far from TS_MAGIC_HEADLIST_FRONT_NUMBER to look for right list_head 
// (both sides,unit: list_heads;)
// See theoretical explanation for more information.

#define TS_MAGIC_MIN_LIST_SIZE 10
// At least how many processes must be in process list

#define TS_MAGIC_MAX_TRIES 350 //200 //160		
// how many times try to find "tasks" inside task_struct..can vary!

#define TS_MAGIC_HL_MM_DST_P 8
// distance from "tasks"	list_head to "mm" mmstruct...can vary!

#define TS_MAGIC_LIST_MAX_ERR 15
// Maximally how many errors can be in head_list

#define TS_MAGIC_FILES_MAX_TRIES 100
// how many times try to find "files" inside task_struct (20 should be enough;)

#define TS_MAGIC_HEADLIST_MAX 20
// Max. how many headlists we expect before comm in task_struct. 

#define TS_COMM_LEN 16
//length of comm array of chars (process name)


// ----- mm_struct -----
#define MM_MAGIC_PGD_DST_P 18
// distance from beginning of mm_struct to pgd variable...can vary!

// ----- module -----
#define MODULE_MAGIC_OPT_ADDR 0x30000000
// optimization when looking for module. 
// usually, modules are beyond 0x3000000, so we can save reading of 1G of data
// If this fail, we perform full scan
// architecture depentent (!)

#define MD_MAGIC_HEADLIST_NUMBER 1
// distance in list_head occurrences between list_head and name in module struct :) 
// actually not used, because they are just near
// TODO add specification by parameter ?

#define MD_MAGIC_MAX_TRIES 1
// how many times try to find "list" inside module..can vary!

#define MD_MAGIC_HL_NAME_DST_P 6
// distance from "list" list_head to "name" in module...can vary!

#define MD_MAGIC_HEADLIST_MAX 10
// Max. how many headlists we expect before name in module

#define MD_NAME_LEN 64
// length of comm array of chars (module name)

#define MD_PATTERN_SPREAD 100
// length of comm array of chars (module name)
// This is needed, because in linux list_head and module name are
// too close to each other

// ----- qstr struct -----
#define QS_MAGIC_MAX_FILENAME_SIZE 512
// max. how long can be filename, when trying to list open files

// ----- dentry struct -----
#define DT_MAGIC_MAX_TRIES 20
// max. how many times try to verify qstr inside dentry struct

// ----- vfs struct -----
#define VFS_MAGIC_MAX_TRIES 20	// max. how many try to verify devname inside vfs_struct
#define VFS_MAGIC_PATTERN  "/dev/"	// pattern we are looking for when verifying vfs_struct
#define VFS_MAGIC_PATTERN_LEN 5 // length of pattern when verifying vfs_struct

// ----- files_struct -----
#define FS_MAGIC_MAX_TRIES 20  //max. how many times try to find fdt inside files_struct

// ----- fdtable struct -----
#define FD_MAGIC_MAX_TRIES 20  //max. how many times try to find fd inside fdtable_struct

// ----- file struct -----
#define FL_MAGIC_MAX_TRIES 20  //max. how many times try to find fdt inside file_struct 

// Maximal number of processes that can found in analysed system
#define MAX_PROC_FOUND	65535

// Maximal size of analysed list
#define MAX_LIST_SIZE	65535

#endif
