#ifndef _I386_FUNCTIONS_H
#define _I386_FUNCTIONS_H

/*
 *	Functions for analysing i386 architecture systems
*/

#include "../../globals.h"
#include "../functions_allarch.h"

#include "magic.h" 

/*
 * Function that manages mapping from linear to physical address space.
 * 
 */
int i386_read_linear(u_int64_t PGD, u_int64_t lin_addr, void *out_data, u_int64_t priv_block_size, u_int64_t *dump_offset);

/*
 * Function for verifying if given address contains VPT 
 * (resp. PaGeDirectory) of some process.
 *
 * Verification is done by simple checking
 * if flags for first three records beyond 0xC0000000
 * (beginning of kernel address space) are same.
 * This should usually be true, as userspace PGD are 
 * cloned from kernel PGD, but it is just heuristic, 
 * and can be wrong.
 *
 *  user_VPT_test (linear address to verify )
 *  return 0 on success, otherwise error > 0
 *
 * */

int i386_user_VPT_test(u_int64_t my_addr);

/*
 * Function for verifying if given address contains system VPT 
 * (resp. PaGeDirectory)
 *
 * Verification is done by simple checking
 * if first VPT_MAGIC_ZEROES bytes contain only zeroes and 
 * if flags for first three records beyond 0xC0000000
 * (beginning of kernel address space) are same.
 * This should usually be true, but it is just heuristic, 
 * and can be wrong.
 *
 *  VPT_test (physical address to verify )
 *  return 0 on success, otherwise error > 0
 *
 * */

int i386_VPT_test(u_int64_t my_addr);


/*
 * Function that tries to determine general information 
 * about analyzed image.
 *
 * for now, only system VPT is found.
 *
 * i386_determine ()
 * return 0 on success, otherwise error > 0
 * Discovered information are saved in global structure image1.
 *
 * */

int i386_determine();

#endif
