/*
 *             foriana
 * 
 *  FOrensic Ram Image ANAlyzer
 *  written in 2007-2010 by niekt0
 *  (niekt0@hysteria.sk, https://hysteria.sk/~niekt0)
 *  (please, mail me any discovered bugs/improvements.)
 * 
 * 
 *  version 0.8.0
 * 
 *  newest version at https://hysteria.sk/~niekt0/foriana
 * 
 *  Under GNU GPL.
 *  See http://gnu.org for more information.
 * 
 *  Use at your own risk.
 * 
*/


/* Limits of this version:
   see TODO
 */

#include "globals.h"
#include "cache.h"
#include <arch/functions_allarch.h>

/*
 *  Global variables
 *  (little bit dirty)
 */


struct my_image image1;

char helpstatusfile[256]="foriana.cache"; //this will go to constants
char *statusfile2,*statusfile=helpstatusfile;
FILE *fs=NULL;		//cache file

char *inputfile;	//filename of analyzed image
char *process_mem_file=NULL;	//filename of output file with memory dump of process 
int inputfd,statusfd;	//input & status output 
FILE *f;		//separate descriptor for searching in file

struct stat sbuffer;    //for input file stats
char *errormsg;

char MAGIC_MODULE[MD_NAME_LEN]=LINUX_MAGIC_MODULE; // user can specify magic constants
char MAGIC_PROCESS[TS_COMM_LEN]=LINUX_MAGIC_PROCESS; //-||-
char MAGIC_USER_PROCESS[TS_COMM_LEN]=LINUX_MAGIC_USER_PROCESS; //-||-


int i,j,opt,mode=1,status=0,tmp;
u_int64_t read_addr=0,size_read_addr=0,pgd_read_addr=0,single_process=0,single_module=0;
unsigned char *pointer;

int do_mod_list=0,do_mod_det=0,do_proc_det=0,do_proc_list=0; //for parsing cmdline options
int do_files_det=0,do_files_list=0,do_arreas_list=1,ignore_cache=0,ddebug=0; // -||-
int do_create_proc_pattern=0,do_create_module_pattern=0;	// -||-
int cache_readed=0;	// -||-
//to save which foriana functionalities user requested

/* 
 * Initialization function
 *
 * return > 0 if error occur
 * otherwise 0
 */

int init(){

	// Open input file	
        if ( (inputfd = open(inputfile,O_RDONLY)) == -1 ) {
                errormsg = strerror(errno);
                fprintf(stderr, "%s: %s.\n",inputfile,errormsg);
                return 21;
        }

	// Get file size
	if ( fstat(inputfd,&sbuffer) == -1){
		fprintf(stderr, "%s: Cannot get file statistics.\n",inputfile);
		return 22;
	}

	image1.ram_size=sbuffer.st_size;
	strncpy(image1.architecture,"unknown\0",9);
	strncpy(image1.operating_system,"unknown\0",9);

	// Check if we can read image "header"
	if (image1.ram_size < MB_1){	
		fprintf(stderr, "%s: File is too small (<1MB). Are you sure it is a memory dump? If yes, please modify foriana source code.\n",inputfile);
		return 23;
	}
	
	printf("Dumped RAM size is %llu MB.\n",image1.ram_size/MB_1);
	
	return 0;

}

void help(){

	printf("\nforiana %s by niekt0@hysteria.sk\n",fversion); 
	printf("Usage: foriana [options] RAM_image\n");
	printf("Options:\n");
	printf("  -a/--all : try to dump everything available. Generally not good idea.\n");
	printf("  --arch <i386,x64,arm> : set architecture manually,disable automatic architecture detection\n");
	printf("  -c/--cache <file> : specify cache file with already determined information\n");

	printf("  --create-process-pattern : try to create pattern that matches all process structures\n"); 
	printf("  --create-module-pattern : try to create pattern that matches all module structures\n");
	printf("  --debug: development debug informations will be printed\n");
	printf("  -d/--determine-only : only determine information about image, do not list anything\n");
	printf("  --determine-skip-processes : skip determination of process structure\n");
	printf("  --determine-skip-modules : skip determination of module structure\n");
	printf("  --determine-skip-files : skip determination of various file structures\n");
	printf("  -h/--help : secret option\n");

	printf("  -i/--ignore-cache : ignores stored cache, and force new determination\n");

	printf("  --list-modules : force listing of modules\n");
	printf("  --list-processes : force listing of processes\n");
	printf("  --list-files : (NOT WORKING NOW) force listing of open files\n");
	printf("  --list-skip-areas : do not list mapped areas for each process\n");

	printf("  --magic-module <name> : specify module name foriana is looking for (default: \"kthread\")\n");
	printf("  --magic-process <name> : specify process name foriana is looking for (default: \"yenta_socket\")\n");
	printf("  --magic-user-process <name> : specify process name foriana assumes first one (default: \"init\")\n");

	printf("  -m/--module <address> : list all information about this module only\n");
	printf("  -p/--process <address> : list all information about this process only\n");
	printf("  --process_mem <file> : dump all mapped memory of this process into file\n");

	printf("  --pgd-read-linear <physical address of process page directory> : kernel PGD by default\n");
	printf("  --pgd <physical address of system page directory> : determined by default\n");
	printf("  --plugins <comma separated list of IDs of plugins to be run>:\n");
	printf("             (options like -d are just aliases for plugin numbers \n");

	printf("  --read-linear <linear address> : liner address to read size-read-linear bytes from\n");
	printf("  --show-plugins : list all available pluggins\n");
	printf("  --system <linux,windows,bsd> : Give foriana hint, what system is analyzed\n");

	printf("  --size-read-linear <size in bytes> : for --read-linear\n");

	printf("  -v/--version : print version number\n");
}

int main(int argc, char **argv)
{	

	// get_parameters

	static struct option long_opts[]={
		{"all",0,0,'a'},
		{"arch",1,0,254},
		{"cache",1,0,'c'},
		{"create-process-pattern",0,0,261},
		{"create-module-pattern",0,0,262},
		{"debug",0,0,248},
		{"determine",0,0,'d'},    
		{"determine-skip-processes",0,0,245},
		{"determine-skip-modules",0,0,246},
		{"determine-skip-files",0,0,247},
		{"help",0,0,'h'},
		{"ignore-cache",0,0,'i'},
		{"list-files",0,0,249},
		{"list-modules",0,0,240},
		{"list-processes",0,0,241},	
		{"list-skip-areas",0,0,256},
		{"module",1,0,'m'},
		{"magic-module",1,0,242},
		{"magic-process",1,0,243},
		{"magic-user-process",1,0,244},
//		{"output",1,0,'o'}, // decided to use stdout instead 
		{"process",1,0,'p'},
		{"process_mem",1,0,260},
		{"pgd-read-linear",1,0,250},
		{"pgd",1,0,270},
		{"plugins",1,0,253},
		{"read-linear",1,0,'r'},
		{"show-plugins",0,0,252},
		{"size-read-linear",1,0,'s'},
		{"system",1,0,251},
		{"version",0,0,'v'},
		{0,0,0,0}                             
	};

	
	/*
	 * Processing parameter in multiple runs is needed,
	 * because some parameters from cache can be overridden,
	 * so cache must be read before this parameter are processed.
	 * Unfortunately, cache filename is one of parameters, so we 
	 * need to extract filename 1 step earlier.
	 *
	 */


//	dbgprint("Terrible hack. I will die in hell for this.");
	// 1.st iteration: If cache file should be ignored.
	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",long_opts, &i)) != -1) {
	 switch(opt) {
		case 'i': /*--ignore-cache*/
			ignore_cache=1; 
			break; /*--ignore-cache*/
	 }
	}

	// Get analyzed filename
        if ((argv[optind] != NULL) && (argv[optind+1]==NULL)) {
                printf("Analyzing file %s\n",argv[optind]);
		inputfile = argv[optind];
        }else {
		fprintf(stderr, "Syntax error.\n"); 
		help();
		return 7;	
	}

	optind=1;

	// 2.st iteration: Read cache file first, and then override user specified values 
	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",long_opts, &i)) != -1) {
	 switch(opt) {
		case 'c': /*--cache*/
			statusfile = optarg;
			break;
	 }
	}
	optind=1;

	/*
	 * Foriana check for cache file named "." + basename() of analysed image 
	 * inside working directory. If no cache file is found, new cache is
	 * created of this name. Name can be overridden with -c option.
	 *
	 */
	//dbgprint("statusfile0: %s\n",statusfile);fflush(stdout); //DBG
	if ( statusfile == helpstatusfile ) {	// cache file not specified
		
		//have to copy string before calling basename, according to man 3 basename
		if ( (statusfile2=strdup(inputfile) ) == NULL) {
			fprintf(stderr, "filename failed.\n");
			return 98;
		}	

		snprintf(statusfile,256,".%s", basename(statusfile2)); //max cache filename size
		free(statusfile2);

	}



	// Read cache content.
	// This must be done before proccessing rest of arguments.

	dbgprint("Reading cache file");
	if ((ignore_cache > 0) || ((cache_readed = read_info()) != 0))
	{
		if (ignore_cache > 0) {
			printf("Ignoring cache file.\n");
		}else{
                	printf("no valid cache found (err %d).\n",cache_readed);
		}
	}

	while((opt = getopt_long(argc, argv, "ac:dhim:p:r:s:v",long_opts, &i)) != -1) {
//		printf("opt: %d\n",opt);
	  switch(opt) {
		case 'a': /*--all*/
			if (!(do_mod_det < 0)) {do_mod_det = 1; }
			if (!(do_mod_list < 0)) {do_mod_list = 1;}
			if (!(do_proc_det < 0)) {do_proc_det = 1;}
			if (!(do_proc_list < 0)) {do_proc_list = 1;}

			if (!(do_files_det < 0)) {do_files_det = 1;}
			if (!(do_files_list < 0)) {do_files_list = 1;}
			break;
		case 254: /*--arch*/
			strncpy(image1.architecture,optarg,32);
			image1.architecture[31]='\0';
			image1.arch=0;
			if (strncmp("i386",optarg,4) == 0) {image1.arch=1;}
			if (strncmp("x64",optarg,3) == 0) {image1.arch=2;}
			if (strncmp("x86_64",optarg,6) == 0) {image1.arch=2;} //alternative name
			if (strncmp("arm",optarg,3) == 0) {image1.arch=3;}
			
			if (image1.arch==0){ // unknown architecture
				printf("Unknown architecture, \"%s\".\n",optarg);
				return 0;
			}

			printf("User specified architecture: %s\n",image1.architecture);
			break;
		case 'c': /*--cache*/
			statusfile = optarg;
			break;
		case 261: /*--create-process-pattern*/
			do_proc_list=1; // listing of processes is required
			do_create_proc_pattern=1;
			break;
		case 262: /*--create-module-pattern*/
			do_mod_list=1; // listing of modules is required
			do_create_module_pattern=1;
			break;
		case 248: /*--debug*/
			ddebug=1;
			break;
		case 'd': /*--determine*/
			if (!(do_mod_det < 0)) {do_mod_det = 1;}
			if (!(do_proc_det < 0)) {do_proc_det = 1;}
			if (!(do_files_det < 0)) {do_files_det = 1;}
			break;
		case 'h': help(); return 0; break;
		case 'i': ignore_cache=1; break; /*--ignore-cache*/
		case 'm': /*--module*/
	 		if (mode == 1) {mode=800; } else {mode=-800;} // --module is solo
			single_module=strtoul(optarg,NULL,16); //hexa
			break; 
//		case 'o': outputfile=optarg; break; // Using stdout instead
		case 253: /*--plugins*/ //TODO
			printf("plugins: TODO\n");
			return 15;
			break;
		case 'p':/*--process*/
	 		if (mode == 1) {mode=900; } else {mode=-900;} // --process is solo
			single_process=strtoul(optarg,NULL,16); //hexa
			break; 
		case 260:/*--process_mem*/
			process_mem_file=optarg;
			break; 
		case 'r': /*--read-linear*/	
			if (mode == 1) {mode=700; } else {mode=-700;}
			read_addr=strtoul(optarg,NULL,16); //hexa
			break;
		case 252: /*--show-plugins"*/
			printf("TODO");
			break;
		case 's': /*--size-read-linear*/	
			size_read_addr=atol(optarg);
			break;
		case 251: /*--system*/	
			strncpy(image1.operating_system,optarg,32);
			image1.operating_system[31]='\0';	
			image1.os=OS_UNKNOWN;
			if (strncmp("linux",optarg,5) == 0) {image1.os=OS_LINUX;}
			if (strncmp("lin",optarg,3) == 0) {image1.os=OS_LINUX;}
			if (strncmp("windows",optarg,7) == 0) {image1.os=OS_WINDOWS;}
			if (strncmp("win",optarg,3) == 0) {image1.os=OS_WINDOWS;}
			if (strncmp("bsd",optarg,3) == 0) {image1.os=OS_BSD;}

			if (image1.os==OS_UNKNOWN){ // unknown os
				printf("Unknown OS, \"%s\".\n",optarg);
				return 0;
			}

			printf("User specified OS: %s\n",image1.operating_system);
			break;
		case 250: /*--pgd-read-linear*/
			pgd_read_addr=strtoul(optarg,NULL,16); //hexa
			break;
		case 270: /*--pgd*/
			image1.VPT_address=strtoul(optarg,NULL,16); //hexa
//			dbgprint("pgd: %0x%llX",image1.VPT_address);
			image1.VPT_user_specified=1;
			if (strncmp("-1",optarg,2) == 0) {
				image1.VPT_not_available=1;
				image1.VPT_address=0;
			}
			if (strncmp("no",optarg,2) == 0) {
				image1.VPT_not_available=1;
				image1.VPT_address=0;
			}
			if (strncmp("hard",optarg,4) == 0) {
				image1.VPT_not_available=1;
				image1.VPT_address=0;
			}
			break;
		case 249: /*--list-files*/
			do_files_list=1; //1 TODO not working right now ;
			break;
		case 240: /*--list_modules*/
			do_mod_list=1;
			break;
		case 241: /*--list-processes*/
			do_proc_list=1;
			break;
		case 256: /*--list-skip-areas*/
			do_arreas_list=0; 
			break;
		case 242: /*--magic-module*/  
//			memcpy(MAGIC_MODULE,optarg,strlen(optarg)+1); //+1 because of \0
			strncpy(MAGIC_MODULE,optarg,MD_NAME_LEN);
			MAGIC_MODULE[MD_NAME_LEN -1]= '\0'; //to be sure
			printf("Using magic_module: %s\n",MAGIC_MODULE); 
			break;
		case 243: /*--magic-process*/
//			memcpy(MAGIC_PROCESS,optarg,strlen(optarg)+1); //+1 because of \0
			strncpy(MAGIC_PROCESS,optarg,TS_COMM_LEN);
			MAGIC_PROCESS[TS_COMM_LEN -1]= '\0'; //to be sure
			printf("Using magic_process: %s\n",MAGIC_PROCESS); 
			break;
		case 244: /*--magic-user-process*/
//			memcpy(MAGIC_USER_PROCESS,optarg,strlen(optarg)+1); //+1 because of \0
			strncpy(MAGIC_USER_PROCESS,optarg,TS_COMM_LEN);
			MAGIC_USER_PROCESS[TS_COMM_LEN -1]= '\0'; //to be sure
			printf("Using magic_user_process: %s\n",MAGIC_USER_PROCESS);
			break;
		case 245: /*--determine-skip-processes*/ 
			do_proc_det=-1; // Conflict flag
			//	do_mod_list=-1; // No listing without determination? 
			// It's less user-friendly, but allow you to modify cache file by hand.
			break;
		case 246: /*--determine-skip-modules*/;
			do_mod_det=-1; // Conflict flag
			//	do_mod_list=-1; // No listing without determination? 
			// It's less user-friendly, but allow you to modify cache file by hand.
			break;
		case 247: /*determine-skip-files*/
			do_files_det=-1; // Conflict flag
			//	do_mod_list=-1; // No listing without determination? 
			// It's less user-friendly, but allow you to modify cache file by hand.
			break;

		case 'v': printf("foriana %s\n",fversion); return 0; break;
		case '?': fprintf(stderr, "Syntax error.\n"); help(); return 7; break;	
	  }   	
	}	



	// parameter defaults

	// parameter collisions 

	/* Following parameter collisions are checked:
	 * - --ignore-cache, and --determine-skip-X and --list-X 
	 * - --read-linear and determine/list option
	 * - --process must be used solo
	 */
	
	if ((do_mod_det < 1) && (do_mod_list == 1) && (ignore_cache == 1))  {
		fprintf(stderr,"Parameter collision: Trying to list modules, but determined information not available...exiting\n");
		return 13;
	}

	if ((do_proc_det < 1) && (do_proc_list == 1) && (ignore_cache == 1))  {
		fprintf(stderr,"Parameter collision: Trying to list processes, but determined information not available...exiting\n");
		return 13;
	}

	if ((do_files_det < 1) && (do_files_list == 1) && (ignore_cache == 1))  {
		fprintf(stderr,"Parameter collision: Trying to list open files, but determined information not available...exiting\n");
		return 13;
	}

	if ((mode > 1) && ((do_mod_list == 1) || (do_files_list == 1) || (do_proc_list == 1) 
	|| (do_mod_det == 1) || (do_files_det == 1) || (do_proc_det == 1)))  { //read_linear
		fprintf(stderr,"Parameter collision: some options must be used solo...exiting\n");
		return 13;
	}

	if ((mode == 700) && (size_read_addr == 0))  { //size_read_linear
		fprintf(stderr,"Parameter collision: size-read-linear not specified...exiting\n");
		return 13;
	}


	// ----------------- program starts here ---------------------
	
		

	//Check size of input file & initialize
	if ((i = init()) != 0) {
		printf("Init returned %d. Exiting.\n",i);
		return 1;
	}


	if (ddebug == 1 ) {debug(); } //DBG
	fflush(stdout);
	//Determine information only if ignore_cache is not set,
	// or if cache file is missing or invalid

//	if ((ignore_cache > 0) || ((i = read_info()) != 0))
	if ((ignore_cache > 0) || cache_readed !=0)
	{
//		if (ignore_cache > 0) {
			
			// Another (runtime) parameter collisions
			if ((do_mod_det < 1) && (do_mod_list == 1) )  {
				fprintf(stderr,"Parameter collision: Trying to list modules, but determined information not available...exiting\n");
				return 14;
			}
			if ((do_proc_det < 1) && (do_proc_list == 1))  {
				fprintf(stderr,"Parameter collision: Trying to list processes, but determined information not available...exiting\n");
				return 14;
			}
			if ((do_files_det < 1) && (do_files_list == 1))  {
				fprintf(stderr,"Parameter collision: Trying to list open files, but determined information not available...exiting\n");
				return 14;
			}
//		}
        	
		// If architecture was not specified
		if (image1.arch == 0) {
			if ( (i=guess_architecture()) ) {
				//something is wrong..
				printf("guess_architecture returned %d. Exiting.\n",i);
				printf("Try to specify architecture manually using --arch option.\n");
				return 0;
			}
		}
		
		// Correctly set all function pointers according to 
		// selected/guessed architecture.
		i=arch_os_settings();

		//Determine general information from image ()
		
		if ((i = image1.determine()) != 0) {
	                printf("Determine returned %d. Exiting.\n",i);
	                return 2;
	        }



		if ( do_proc_det > 0 ) {
			if ((i = proc_determine()) != 0) {
	        		printf("Linux_determine returned %d. Exiting.\n",i);
       				return 3;
			}
        	}

		if ( do_mod_det > 0 ) {
			if ((i = lin_linux_determine_module()) != 0) {
        	       		printf("Linux_determine_module returned %d. Exiting.\n",i);
                		return 4;
        		}
		}

		if ( do_files_det > 0 ) {
			printf("Determine open files: TODO (still bugy)\n");
//		        if ((i = i386_lin_determine_open_files(image1.ts_one_of_lists)) != 0) {
//				printf("i386_lin_determine_open_files returned %d. Exiting.\n",i);
//				return 6;
//			}
		}

		if ((i = save_info()) != 0) {
	                printf("save_info returned %d. Exiting.\n",i);
	                return 5;
	        }
		search_exit(); //for closing open file

	}
	fflush(stdout);
	
//	i=lin_verify_task_struct(0xccd6b1c8);

	/* -p/--process <hex_addr>
	 * Display: 
	 * - process linear address
	 * - process physical address
	 * - process name
	 * - process PGD (for --read-linear)
	 * - list of open files for this process
	 */
	if (mode == 900) { // --process is solo
		// Put this into function (for multiple architectures))
		if (single_process == 0) {
			printf("Invalid process address.\n");
			return 902;
		}
	
		return print_process_info(single_process,process_mem_file);		
	}

	/* -m/--module <hex_addr>
	 * Display: 
	 * - module address
	 * - module physical address
	 * - module name
	 * - ?
	 */
	if (mode == 800) { // --module is solo
		if (single_module == 0) {
			printf("Invalid module address.\n");
			return 802;
		}

		return print_module_info(single_module);

	}

	/* --read-linear <hex_addr>
	 * Display: 
	 * - linear address
	 * - physical address (first block)
	 * - hex dump
	 * - ascii dump (printable chars)
	 */
	if (mode == 700) { // read_linear is solo

		u_int64_t tmp700=0;
		
		printf("Reading %llu bytes from linear address 0x%llx.\n",size_read_addr,read_addr);
		if ((pointer=(unsigned char *)malloc(size_read_addr)) == NULL) { // Not ideal solution for big blocks...
			fprintf(stderr, "malloc() failed.\n");
			return 724;
		}

		// if not specified, pgd_read_addr is 0 (kernel pgd)
		if ( (i = image1.read_linear(pgd_read_addr,read_addr, pointer,size_read_addr,&tmp700) == -1 )) {
       	        	printf("--read-linear: 0x%llx seems is not valid virtual address.\n",read_addr);
			free(pointer);
    	       		return(723);
	       	} else {
			if (tmp700 == 0) { //not mapped
				printf("0x%llx does not seem to be mapped to physical memory.\n",read_addr);
				free(pointer);
				return(742);
			} else {
				printf("Read %llu bytes from 0x%llx (physical address 0x%llx):\n",size_read_addr,read_addr,tmp700);
			}
		}
		
		// now print block we have read
		printf("Hex dump & printable strings:\n");
		u_int64_t utmp=0;
		utmp=((LINE_SIZE/4)-1)& ~3; 
		//for each character, hexa number, (2 chars) space (1 char) and char itself (1 char) is printed
		//on standard 80 chars terminal, this makes 16 bytes per line
		
		//print loop
		printf("%llx: ",read_addr);
		u_int64_t ui=0,uj=0;
		for (ui=0,uj=0;ui<size_read_addr;ui++) {
			printf("%02x ",pointer[ui]);
			uj++;
			if ((uj == utmp) || (ui == size_read_addr-1)) { //end of line
				if (ui == size_read_addr-1) { //special case: last line
					int tmp3=uj;
					for (;tmp3<tmp;tmp3++) {printf("   ");} 
				}
				u_int64_t ui2=ui-uj+1;
				putc(' ',stdout);
				for(;ui2<=ui;ui2++) {
					int tmp2=pointer[ui2];
					if (isprint(tmp2) && ( tmp2 == ' ' || !isspace(tmp2)) ) {
					//only printable chars,but not end of lines,but yes space;)
						putc(tmp2,stdout);
					} else {
						putc('.',stdout); //non printable (in ascii)
					}
				}
				//printf("%d",i);
				printf("\n%llx: ",read_addr+ui+1);
				uj=0;
			}
		}	
		printf("End of hexdump.\n");	
		free(pointer);
		return 0;
	}

	//Listing stuff
	
	if ( do_proc_list > 0 ) {
		if ((i = list_processes(image1.ts_one_of_lists,image1.ts_hl_comm_dst)) != 0) {
        	        printf("Listed %d processes (hopefully).\n",i);
		}
        }

 	if ( do_mod_list > 0 ) {
		if ((i = lin_list_modules(image1.md_one_of_lists,image1.md_hl_name_dst)) != 0) {
        	        printf("Listed %d modules (hopefully).\n",i);
        	}
	}

 	if ( do_files_list > 0 ) {
		printf("List open files: TODO (not working)\n"); // TODO
//		if ((i = lin_list_files(image1.md_one_of_lists,image1.md_hl_name_dst)) != 0) {
//        	        printf("Listed %d open files (hopefully).\n",i);
//        	}
	}

	printf("Clean exit;).\n");
	return 0;

}
