#ifndef _STRUCTS_H
#define _STRUCTS_H

#include "arch/allarch.h"

// Typedefs for function.
// We need this, because each architecture have different low-level routines,
// and target architecture is determined on-the-fly

typedef  int (*read_linear_t)(u_int64_t PGD, u_int64_t lin_addr, void *out_data, u_int64_t priv_block_size, u_int64_t *dump_offset);
typedef  int (*VPT_test_t)(u_int64_t my_addr);
typedef  int (*user_VPT_test_t)(u_int64_t my_addr);
typedef  int (*determine_t)();

struct my_image{	// information about file, we are adding
	int guest_block_size;		// size of block in analyzed image
	char architecture[32];		// hw architecture of analyzed image
					// This is only string for user
	int arch;			// -||-, but number representation (for case)
	/* 0 - unknown
	 * 1 - i386
	 * 2 - x86_64
	 * 3 - arm
	 */
	char operating_system[32];	// operating system if analyzed image
					// This is only string for user
	int os;				// -||-, but number representation (for case)
	/* 0 - unknown
	 * 1 - Linux
	 * 2 - Windows 
	 * 10 - Linux/Windows (or other, important is list_head format)
	 * 11 - BSD (different list_head format)
	 */
	u_int64_t VPT_address;	// position of VPT (from beginning of file)
	int VPT_not_available;	// if true, value of VPT_address is ignored, and hard coded values are used
	int VPT_user_specified;	// if true, PGD value was supplied by user, therefore determination is suppressed 
	u_int64_t ram_size;	// size of image in kilobytes
	int ts_hl_comm_dst;	  // distance between comm and task headlist fields in task_struct
	int md_hl_name_dst;	  // distance between name and modules headlist fields in module
	u_int64_t ts_one_of_lists; // address of task list_head of one of taks_struct structures
	u_int64_t md_one_of_lists; // address of list list_head of one of module structures
	int ts_hl_mm_dst; // =TS_MAGIC_HL_MM_DST; // Distance from "tasks" list_head to "mm" mmstruct...can vary!
	int mm_pgd_dst; // =MM_MAGIC_PGD_DST;  //Distance from beginning of mm_struct to pgd variable...can vary!
	int vm_size_dst;	// Distance from beginning of vm_area struct to sizes of VMA
	int vm_list_dst;		// Distance from beginning of vm_area struct to list_head

	int ts_headlists_number;  // how many headl_lists there are in task_struct before comm
	int md_headlists_number;  // how many headl_lists there are in module before name
	u_int64_t ts_headlists[TS_MAGIC_HEADLIST_MAX]; // pointers to headlist of task_struct of one process
	u_int64_t md_headlists[MD_MAGIC_HEADLIST_MAX]; // pointers to headlist of module

	
// stuff for listing of open files:

// => task_struct ->  files   ->    fdt   ->  fd    ->  f_path  ->  mnt
//                (files_struct)  (fdtable)  (file **)   (path)   (vfsmount)
//                                                              ->  dentry -> d_name -> name
//                                                                 (dentry)   (qstr)  (char)

	int ts_hl_fs_dst;	//Distance from "tasks" list_head to "files" files_struct in task_struct
	int fs_fdt_dst;		//Distance from beginning of structure to "fdt" fdtable in files_struct
	int fdt_fd_dst;		//Distance from beginning of structure to "fd" **file in fdtable
	int fl_fpath_dst;	//Distance from beginning of structure to "f_path" path in file structure
	int pt_dentry_dst;	//Distance from beginning of structure to "dentry" dentry in path structure
	int dt_name_dst;	//Distance from beginning of structure to "d_name" qstr in dentry structure
	int qs_name_dst;	//Distance from beginning of structure to "name" *char in qstr. note: Should not change, this structure is elemental
	int fs_vfs_dst;		//Distance from beginning of structure to vfsmount in fs_struct.
	u_int64_t one_of_paths;		//Pointer to one of path structs

// This part is added later, I have put it here, to be clear it is new
	int pointer_size;	// Pointer size of analyzed architecture, determined on-the-fly
	int integer_size;	// Integer size of analyzed architecture, determined on-the-fly

	read_linear_t read_linear; //Here should be stored address of {arch}_read_linear function
	VPT_test_t VPT_test; //Here should be stored address of {arch}_VPT_test function
	user_VPT_test_t user_VPT_test; //Here should be stored address of {arch}_user_VPT_test function
	determine_t determine; //Here should be stored address of {arch}_determine function
};

#endif

